<?php
/**
 * Whups mail processing/generation library.
 *
 * $Horde: whups/lib/Mail.php,v 1.39 2008/02/28 14:19:52 jan Exp $
 *
 * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Jason M. Felice <jason.m.felice@gmail.com>
 * @package Whups
 */
class Whups_Mail {

    /**
     * Parse a MIME message and create a new ticket.
     *
     * @param string $text  This is the full text of the MIME message.
     * @param array $info   An array of information for the new ticket. This
     *                      should include:
     *                        'queue'    => queue id
     *                        'type'     => type id
     *                        'state'    => state id
     *                        'priority' => priority id
     *                        'ticket'   => ticket id (prevents creation of
     *                                      new tickets)
     * @param string $user  This will be the ticket's requester. If null, we
     *                      will try to deduce from the message's "From"
     *                      field--we do NOT default to Auth::getAuth().
     *
     * @return Whups_Ticket | PEAR_Error  Ticket or Error object.
     */
    function processMail($text, $info, $user = null)
    {
        global $conf;

        require_once 'Horde/MIME/Structure.php';
        $message = MIME_Structure::parseTextMIMEMessage($text);

        if (preg_match("/^(.*?)\r?\n\r?\n/s", $text, $matches)) {
            $hdrText = $matches[1];
        } else {
            $hdrText = $text;
        }
        $headers = MIME_Structure::parseMIMEHeaders($hdrText, false);

        // If this message was generated by Whups, don't process it.
        if (!empty($headers['X-Whups-Generated'])) {
            return true;
        }

        // Use the message subject as the ticket summary.
        $info['summary'] = Whups_Mail::_findSimpleHeaderValue($headers, 'Subject');
        $info['comment'] = Whups_Mail::_buildComment($message, $headers);

        if (is_null($user)) {
            $user = Whups_Mail::_findAuthUser($headers);
        }
        if (is_null($user) && !empty($info['default_auth'])) {
            $user = $info['default_auth'];
        }

        if (!is_null($user)) {
            if ($user != Auth::getAuth()) {
                $params = Horde::getDriverConfig('auth', $conf['auth']['driver']);
                $auth = &Auth::singleton($conf['auth']['driver'], $params);
                $auth->setAuth($user, array());
            }
            // FIXME: Use Identity to set email address.
        } else {
            if (!empty($conf['mail']['username'])) {
                $params = Horde::getDriverConfig('auth', $conf['auth']['driver']);
                $auth = &Auth::singleton($conf['auth']['driver'], $params);
                $auth->setAuth($conf['mail']['username'], array());
            }
            $from = Whups_Mail::_findSimpleHeaderValue($headers, 'From');
            if (!empty($from)) {
                $info['user_email'] = $from;
            }
        }

        if ($ticket = Whups_Mail::_findTicket($info)) {
            $ticket->change('comment', $info['comment']);
            $result = $ticket->commit();
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        } elseif (!empty($info['ticket'])) {
            return PEAR::raiseError(sprintf(_("Could not find ticket \"%s\"."),
                                            $info['ticket']));
        } else {
            if (!empty($info['guess-queue'])) {
                $queues = $GLOBALS['whups_driver']->getQueues();
                foreach ($queues as $queueId => $queueName) {
                    if (preg_match('/\b' . preg_quote($queueName, '/') . '\b/i',
                                   $info['summary'])) {
                        $info['queue'] = $queueId;
                        break;
                    }
                }
            }
            $ticket = Whups_Ticket::newTicket($info, $user);
            if (is_a($ticket, 'PEAR_Error')) {
                return $ticket;
            }
        }

        // Extract attachments.
        $attachments = Whups_Mail::_getDownloadAllList($message);
        foreach ($attachments as $attachment) {
            $part = &$message->getPart($attachment);
            $part->transferDecodeContents();
            $tmp_name = Horde::getTempFile('whups');
            $fp = @fopen($tmp_name, 'wb');
            if (!$fp) {
                Horde::logMessage(sprintf('Cannot open file %s for writing.', $tmp_name), __FILE__, __LINE__, PEAR_LOG_ERR);
                return $ticket;
            }
            fwrite($fp, $part->getContents());
            fclose($fp);
            $part_name = $part->getName(true, true);
            if (empty($part_name)) {
                $part_name = MIME_DEFAULT_DESCRIPTION;
            }
            $ticket->change('attachment', array('name' => $part_name,
                                                'tmp_name' => $tmp_name));
            $result = $ticket->commit();
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }
    }

    function _findTicket($info)
    {
        if (!empty($info['ticket'])) {
            $ticketnum = $info['ticket'];
        } elseif (preg_match('/\[[\w\s]*#(\d+)\]/', $info['summary'], $matches)) {
            $ticketnum = $matches[1];
        } else {
            return false;
        }

        $ticket = Whups_Ticket::makeTicket($ticketnum);
        if (is_a($ticket, 'PEAR_Error')) {
            return false;
        }

        return $ticket;
    }

    function _findSimpleHeaderValue($headers, $name)
    {
        foreach ($headers as $hdrName => $hdrVals) {
            if (strcasecmp($hdrName, $name) == 0) {
                if (is_array($hdrVals)) {
                    return implode(', ', $hdrVals);
                } else {
                    return MIME::decode($hdrVals);
                }
            }
        }

        return null;
    }

    function _buildComment(&$message, $headers)
    {
        // Format the message into a comment.
        $comment = _("Received message:") . "\n\n";
        foreach ($headers as $name => $vals) {
            if (!strcasecmp($name, 'Subject') || !strcasecmp($name, 'From') ||
                !strcasecmp($name, 'To') || !strcasecmp($name, 'Cc') ||
                !strcasecmp($name, 'Date')) {
                if (is_array($vals)) {
                    foreach ($vals as $val) {
                        $comment .= $name . ': ' . $val . "\n";
                    }
                } else {
                    $comment .= $name . ': ' . MIME::decode($vals) . "\n";
                }
            }
        }

        $comment .= "\n";

        // Look for the body part.
        $body_id = Whups_Mail::findBody($message);
        if ($body_id) {
            $part = &$message->getPart($body_id);
            $comment .= $part->transferDecode();
        } else {
            $comment .= _("[ Could not render body of message. ]");
        }

        return $comment . "\n";
    }

    /**
     * Generates the list of MIME IDs for all "valid" attachments.
     *
     * @param MIME_Message $message  The MIME_Message object for the message.
     *
     * @return array  The list of MIME IDs that should be considered
     *                attachments.
     */
    function _getDownloadAllList($message)
    {
        $downloads = array();
        $bodyid = Whups_Mail::findBody($message);

        /* Here is what we consider 'downloadable':
         * All parts not 'multipart/*' and 'message/*' except for
         *  'message/rfc822'
         * All parts that are not PGP or S/MIME signature information
         * NOT the body part (if one exists) */
        foreach ($message->contentTypeMap() as $key => $val) {
            if ($key === $bodyid) {
                continue;
            }

            $mime_part = &$message->getPart($key);

            if (strpos($val, 'message/') === 0) {
                if (strpos($val, 'message/rfc822') === 0) {
                    $downloads[] = $key;
                }
            } elseif ((($mime_part->getDisposition() == 'attachment') ||
                       $mime_part->getContentTypeParameter('name')) &&
                      (strpos($val, 'multipart/') === false) &&
                      (strpos($val, 'application/x-pkcs7-signature') === false) &&
                      (strpos($val, 'application/pkcs7-signature') === false)) {
                $downloads[] = $key;
            }
        }

        return $downloads;
    }

    /**
     * Finds the main "body" text part (if any) in a message.
     * "Body" data is the first text part in the base MIME part.
     *
     * @param MIME_Message $message  The MIME_Message object for the message.
     * @param string $subtype        Specifically search for this subtype.
     *
     * @return string  The MIME ID of the main "body" part.
     */
    function findBody($message, $subtype = null)
    {
        /* Look for potential body parts. */
        $part = $message->getBasePart();
        $primary_type = $part->getPrimaryType();
        if (($primary_type == MIME::type(TYPEMULTIPART)) ||
            ($primary_type == MIME::type(TYPETEXT))) {
            return Whups_Mail::_findBody($message, $part, $subtype);
        }

        return null;
    }

    /**
     * Processes a MIME Part and looks for "body" data.
     *
     * @access private
     *
     * @return string  The MIME ID of the main "body" part.
     */
    function _findBody($message, $mime_part, $subtype)
    {
        if (intval($mime_part->getMIMEId()) < 2 ||
            $mime_part->getInformation('alternative') === 0) {
            if ($mime_part->getPrimaryType() == MIME::type(TYPEMULTIPART)) {
                $parts = $mime_part->getParts();
                while ($part = array_shift($parts)) {
                    if (($partid = Whups_Mail::_findBody($message, $part, $subtype))) {
                        return $partid;
                    }
                }
            } elseif ($mime_part->getPrimaryType() == MIME::type(TYPETEXT)) {
                if ($mime_part->getDisposition() != 'attachment' &&
                    (($subtype === null) ||
                     ($subtype == $mime_part->getSubType())) &&
                    ($mime_part->getBytes() ||
                     $message->getPart($mime_part->getMIMEId()))) {
                    return $mime_part->getMIMEId();
                }
            }
        }

        return null;
    }

    /**
     * Search the 'From' header for an email address contained in one
     * of our users' identities.
     *
     * @access private
     */
    function _findAuthUser($headers)
    {
        global $conf;

        require_once 'Horde/Identity.php';
        require_once 'Horde/MIME.php';

        $from = Whups_Mail::_findSimpleHeaderValue($headers, 'From');
        $from = MIME::bareAddress($from);

        $params = Horde::getDriverConfig('auth', $conf['auth']['driver']);
        $auth = &Auth::singleton($conf['auth']['driver'], $params);
        if ($auth->hasCapability('list')) {
            $users = $auth->listUsers();
            foreach ($users as $user) {
                $identity = &Identity::singleton('none', $user);
                $addrs = $identity->getAll('from_addr');
                foreach ($addrs as $addr) {
                    if (strcasecmp($from, $addr) == 0) {
                        return $user;
                    }
                }
            }
        }

        return null;
    }

}
