<?php
/**
 * $Horde: whups/lib/Query.php,v 1.63 2007/06/28 18:05:40 chuck Exp $
 *
 * Copyright 2001-2007 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

/** Horde_Share */
require_once 'Horde/Share.php';

/** Horde_Form_Action */
require_once 'Horde/Form/Action.php';

/** Mode types. */
define('QUERY_TYPE_AND',       1);
define('QUERY_TYPE_OR',        2);
define('QUERY_TYPE_NOT',       3);
define('QUERY_TYPE_CRITERION', 4);

/** Criterion types. */
define('CRITERION_ID',             1);
define('CRITERION_QUEUE',          2);
define('CRITERION_TYPE',           3);
define('CRITERION_STATE',          4);
define('CRITERION_PRIORITY',       5);
define('CRITERION_OWNERS',         7);
define('CRITERION_GROUPS',         9);
define('CRITERION_REQUESTER',      8);
define('CRITERION_ADDED_COMMENT', 11);
define('CRITERION_COMMENT',       12);
define('CRITERION_SUMMARY',       13);
define('CRITERION_ATTRIBUTE',     14);
define('CRITERION_VERSION',       15);

/** Operators for integer fields. */
define('OPERATOR_GREATER', 1);
define('OPERATOR_LESS',    2);
define('OPERATOR_EQUAL',   3);

/** Operators for text fields. */
define('OPERATOR_CI_SUBSTRING',  4);
define('OPERATOR_CS_SUBSTRING',  5);
define('OPERATOR_WORD',          6);
define('OPERATOR_PATTERN',       7);

$textOperators = array(
    OPERATOR_EQUAL        => _("Exact Match"),
    OPERATOR_CI_SUBSTRING => _("Case Insensitive Substring"),
    // OPERATOR_CS_SUBSTRING => _("Case Sensitive Substring"),
    OPERATOR_WORD         => _("Match Word"),
    OPERATOR_PATTERN      => _("Match Pattern"));

/**
 * array(
 *     'type'      => QUERY_TYPE_...
 *     'children'  => array(...) unless type == QUERY_TYPE_CRITERION
 *     'criterion' => CRITERION_... if  type == QUERY_TYPE_CRITERION
 *     'operator'  => OPERATOR_...  if  type == QUERY_TYPE_CRITERION
 *     'value'     => other argument to operator of criterion
 */

/**
 * @package Whups
 */
class Whups_Query {

    /**
     * @var Whups_QueryManager
     */
    var $_qManager;

    /**
     * Query id.
     *
     * @var integer
     */
    var $id;

    /**
     * @var string
     */
    var $name;

    /**
     * @var array
     */
    var $query = array('type' => QUERY_TYPE_AND,
                       'children' => array());

    /**
     * @var array
     */
    var $parameters = array();

    /**
     * @static
     */
    function pathToString(&$path)
    {
        return implode(',', $path);
    }

    /**
     * @static
     */
    function stringToPath($pathstring)
    {
        if (strlen($pathstring) == 0) {
            return array();
        }

        return explode(',', $pathstring);
    }

    /**
     * Constructor
     */
    function Whups_Query(&$qManager, $qDetails = array())
    {
        $this->_qManager = &$qManager;
        if (isset($qDetails['query_id'])) {
            $this->id = $qDetails['query_id'];
        }
        if (isset($qDetails['query_name'])) {
            $this->name = $qDetails['query_name'];
        }
        if (isset($qDetails['query_object'])) {
            $this->query = @unserialize($qDetails['query_object']);
        }
        if (isset($qDetails['query_parameters'])) {
            $this->parameters = @unserialize($qDetails['query_parameters']);
        }
    }

    /**
     * Checks to see if a user has a given permission.
     *
     * @param string $userid       The userid of the user.
     * @param integer $permission  A PERMS_* constant to test for.
     * @param string $creator      The creator of the event.
     *
     * @return boolean  Whether or not $userid has $permission.
     */
    function hasPermission($userid, $permission, $creator = null)
    {
        return $this->_qManager->hasPermission($this->id, $userid, $permission, $creator);
    }

    /**
     * Saves any changes to this object to the backend
     * permanently. New objects are added instead.
     *
     * @return boolean|PEAR_Error  PEAR_Error on failure.
     */
    function save()
    {
        return $this->_qManager->save($this);
    }

    /**
     * Delete this object from the backend permanently.
     *
     * @return boolean|PEAR_Error  PEAR_Error on failure.
     */
    function delete()
    {
        return $this->_qManager->delete($this);
    }

    /**
     * Tab operations for this query.
     */
    function getTabs($vars)
    {
        // Create a few variables that are reused.
        $queryurl = Horde::applicationUrl('query.php');
        $edit = $this->hasPermission(Auth::getAuth(), PERMS_EDIT);
        $delete = $this->hasPermission(Auth::getAuth(), PERMS_DELETE);

        $tabs = new Horde_UI_Tabs('action', $vars);
        $tabs->addTab(_("Ne_w Query"), $queryurl, 'new');
        if (!$this->id || $edit) {
            $tabs->addTab(_("_Edit Query"), $queryurl, 'edit');
        }
        if ($this->id && $edit && empty($GLOBALS['conf']['shares']['no_sharing'])) {
            Horde::addScriptFile('popup.js', 'horde', true);

            $permsurl = $GLOBALS['registry']->get('webroot', 'horde') . '/services/shares/edit.php';
            $permsurl = Util::addParameter($permsurl, array('app' => 'whups',
                                                            'cid' => $this->id));
            $tabs->addTab(_("Edit _Permissions"), $permsurl, array('tabname' => 'perms',
                                                                   'onclick' => 'popup(\'' . $permsurl . '\'); return false;',
                                                                   'target' => '_blank'));
        }
        $tabs->addTab(_("E_xecute Query"), Horde::applicationUrl('runquery.php'), 'run');
        $tabs->addTab(_("_Load Query"), $queryurl, 'load');
        if ((!$this->id && Auth::getAuth()) ||
            ($this->id && $edit)) {
            $tabs->addTab(_("Sa_ve Query"), $queryurl, 'save');
        }
        if ($this->id && $delete) {
            $tabs->addTab(_("_Delete Query"), $queryurl, 'delete');
        }

        return $tabs;
    }

    /**
     * Operations.
     */
    function deleteNode($pathstring)
    {
        $path = Whups_Query::stringToPath($pathstring);
        $qobj = &$this->query;

        if (!strlen($pathstring)) {
            // Deleting the root node isn't supported.
            $GLOBALS['notification']->push(_("Choose New Query instead of deleting the root node."), 'horde.warning');
            return false;
        } else {
            $count = count($path) - 1;
            for ($i = 0; $i < $count; $i++) {
                $qobj = &$qobj['children'][$path[$i]];
            }

            if (!empty($qobj['children'][$path[$count]]['value']) &&
                $this->_getParameterName($qobj['children'][$path[$count]]['value']) !== null) {
                unset($this->parameters[array_search($pn, $this->parameters)]);
            }

            array_splice($qobj['children'], $path[$count], 1);
        }
    }

    function hoist($pathstring)
    {
        $path = Whups_Query::stringToPath($pathstring);
        $qobj = &$this->query;

        if (strlen($pathstring) == 0) {
            // Can't hoist the root node.
        } else {
            $count = count($path) - 1;

            for ($i = 0; $i < $count; $i++) {
                $qobj = &$qobj['children'][$path[$i]];
            }

            $cobj = &$qobj['children'][$path[$count]];

            // TODO: make sure we're hoisting a branch.
            array_splice($qobj['children'], $path[$count], 0, $cobj['children']);
            array_splice($cobj['children'], 0, count($cobj['children']));
        }
    }

    function insertBranch($pathstring, $type)
    {
        $path = Whups_Query::stringToPath($pathstring);
        $qobj = &$this->query;

        $newbranch = array(
            'type'     => $type,
            'children' => array());

        $count = count($path);

        for ($i = 0; $i < $count; $i++) {
            $qobj = &$qobj['children'][$path[$i]];
        }

        if (!isset($qobj['children']) ||
            !is_array($qobj['children'])) {
            $qobj['children'] = array();
        }

        $qobj['children'][] = $newbranch;
        $path[] = count($qobj['children']) - 1;

        return Whups_Query::pathToString($path);
    }

    function insertCriterion($pathstring, $criterion, $cvalue, $operator, $value)
    {
        $path = Whups_Query::stringToPath($pathstring);
        $qobj = &$this->query;

        $value = trim($value);
        if ($value[0] == '"') {
            // FIXME: The last character should be '"' as well.
            $value = substr($value, 1, -1);
        } else {
            $pn = $this->_getParameterName($value);
            if ($pn !== null) {
                $this->parameters[] = $pn;
            }
        }

        $newbranch = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => $criterion,
            'cvalue'    => $cvalue,
            'operator'  => $operator,
            'value'     => $value);

        $count = count($path);
        for ($i = 0; $i < $count; $i++) {
            $qobj = &$qobj['children'][$path[$i]];
        }

        $qobj['children'][] = $newbranch;
    }

    /**
     * Top down traversal.
     */
    function walk(&$obj, $method)
    {
        $path = array();
        $more = array();
        $this->_walk($this->query, $more, $path, $obj, $method);
    }

    /**
     * @access private
     */
    function _walk(&$node, &$more, &$path, &$obj, $method)
    {
        if ($node['type'] == QUERY_TYPE_CRITERION) {
            $obj->$method($more, $path, QUERY_TYPE_CRITERION, $node['criterion'],
                          $node['cvalue'], $node['operator'], $node['value']);
        } else {
            $obj->$method($more, $path, $node['type'], null, null, null, null);
        }

        if (isset($node['children'])) {
            $count = count($node['children']);

            for ($i = 0; $i < $count; $i++) {
                $path[] = $i;
                $more[] = ($i < $count - 1);
                $this->_walk($node['children'][$i], $more, $path, $obj, $method);
                array_pop($more);
                array_pop($path);
            }
        }
    }

    /**
     * Bottom up traversal.
     */
    function reduce(&$obj, $method, &$vars)
    {
        return $this->_reduce($this->query, $obj, $method, $vars);
    }

    /**
     * @access private
     */
    function _reduce(&$node, &$obj, $method, &$vars)
    {
        $args = array();

        if (isset($node['children'])) {
            $count = count($node['children']);

            for ($i = 0; $i < $count; $i++) {
                $result = $this->_reduce($node['children'][$i], $obj, $method, $vars);
                $args[] = $result;
            }
        }

        if ($node['type'] == QUERY_TYPE_CRITERION) {
            $value = $node['value'];

            $pn = $this->_getParameterName($value);
            if ($pn !== null) {
                $value = $vars->get($pn);
            }

            $result = $obj->$method($args, QUERY_TYPE_CRITERION, $node['criterion'],
                                    $node['cvalue'], $node['operator'], $value);
        } else {
            $result = $obj->$method($args, $node['type'], null, null, null, null);
        }

        return $result;
    }

    /**
     * @access private
     */
    function _getParameterName($value)
    {
        if (strcmp(substr($value, 0, 2), '${'))
            return null;

        $pn = substr($value, 2, -1);
        if (!is_string($pn))
            $pn = null;

        return $pn;
    }

}

/**
 * @package Whups
 */
class Whups_QueryManager {

    /**
     * Horde_Share instance for managing shares.
     *
     * @var Horde_Share
     */
    var $_shareManager;

    /**
     * Constructor.
     */
    function Whups_QueryManager()
    {
        $this->_shareManager = &Horde_Share::singleton('whups');
    }

    /**
     * Fetch a specific query.
     */
    function getQuery($queryId)
    {
        $share = $this->_shareManager->getShareById($queryId);
        if (is_a($share, 'PEAR_Error')) {
            return $share;
        }

        $queryDetails = $GLOBALS['whups_driver']->getQuery($queryId);
        if (is_a($queryDetails, 'PEAR_Error')) {
            return $queryDetails;
        }

        $queryDetails['query_id'] = $share->getId();
        $queryDetails['query_name'] = $share->get('name');
        return new Whups_Query($this, $queryDetails);
    }

    /**
     * Checks to see if a user has a given permission to $queryId.
     *
     * @param integer $queryId     The query to check.
     * @param string $userid       The userid of the user.
     * @param integer $permission  A PERMS_* constant to test for.
     * @param string $creator      The creator of the event.
     *
     * @return boolean  Whether or not $userid has $permission.
     */
    function hasPermission($queryId, $userid, $permission, $creator = null)
    {
        $share = $this->_shareManager->getShareById($queryId);
        if (is_a($share, 'PEAR_Error')) {
            // If the share doesn't exist yet, then it has open perms.
            return true;
        }
        return $share->hasPermission($userid, $permission, $creator);
    }

    /**
     * List queries.
     */
    function listQueries($user)
    {
        $shares = $this->_shareManager->listShares($user);
        if (is_a($shares, 'PEAR_Error')) {
            return $shares;
        }

        $queries = array();
        foreach ($shares as $share) {
            $queries[$share->getId()] = $share->get('name');
        }

        return $queries;
    }

    /**
     */
    function newQuery()
    {
        return new Whups_Query($this);
    }

    /**
     * @param Whups_Query $query The query to save.
     */
    function save($query)
    {
        if ($query->id) {
            // Query already exists; get its share and update the name
            // if necessary.
            $share = $this->_shareManager->getShareById($query->id);
            if (is_a($share, 'PEAR_Error')) {
                // Share has an id but doesn't exist; just throw an
                // error.
                return $share;
            }
            if ($share->get('name') != $query->name) {
                $share->set('name', $query->name);
                $share->save();
            }
        } else {
            // Create a new share for the query.
            $share = $this->_shareManager->newShare(md5(microtime()));
            $share->set('name', $query->name);
            $result = $this->_shareManager->addShare($share);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $query->id = $share->getId();
        }

        // Update the queries table.
        $GLOBALS['whups_driver']->saveQuery($query);
    }

    /**
     * @param Whups_Query $query The query to delete.
     */
    function delete($query)
    {
        if (!$query->id) {
            // Queries that aren't saved yet shouldn't be able to be deleted.
            return;
        }

        $share = $this->_shareManager->getShareById($query->id);
        if (is_a($share, 'PEAR_Error')) {
            return $share;
        }

        $result = $this->_shareManager->removeShare($share);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $result = $GLOBALS['whups_driver']->deleteQuery($query->id);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return true;
    }

}

/**
 * Forms for editing queries.
 *
 * @package Whups
 */
class InsertBranchForm extends Horde_Form {

    function InsertBranchForm(&$vars)
    {
        parent::Horde_Form($vars, _("Insert Branch"));

        $branchtypes = array(
            QUERY_TYPE_AND => _("And"),
            QUERY_TYPE_OR  => _("Or"),
            QUERY_TYPE_NOT => _("Not"));

        $this->addHidden(null, 'path', 'text', false, true);
        $this->addVariable(_("Branch Type"), 'type', 'enum', true, false, null, array($branchtypes));
    }

}

/**
 * @package Whups
 */
class UserCriterionForm extends Horde_Form {

    function UserCriterionForm(&$vars)
    {
        global $textOperators;

        parent::Horde_Form($vars, _("Add User Criterion"));

        $this->addVariable(_("User ID"), 'user', 'text', true);
        $this->addVariable(_("Match Operator"), 'operator', 'enum', true, false, null, array($textOperators));
        $this->addVariable(_("Search Owners"), 'owners', 'boolean', false);
        $this->addVariable(_("Search Requester"), 'requester', 'boolean', false);
        $this->addVariable(_("Search Comments"), 'comments', 'boolean', false);
    }

    function execute(&$vars)
    {
        $path = $vars->get('path');
        $user = $vars->get('user');
        $operator = $vars->get('operator');
        $owners = $vars->get('owners');
        $requester = $vars->get('requester');
        $comments = $vars->get('comments');

        // If we're adding more than one criterion, put them all under
        // an OR node (which should be what is wanted in the general
        // case).
        if ((int)$owners + (int)$requester + (int)$comments > 1) {
            $path = $GLOBALS['whups_query']->insertBranch($path, QUERY_TYPE_OR);
        }

        if ($owners) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_OWNERS, null, $operator, $user);
        }

        if ($requester) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_REQUESTER, null, $operator, $user);
        }

        if ($comments) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_ADDED_COMMENT, null, $operator, $user);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class GroupCriterionForm extends Horde_Form {

    function GroupCriterionForm(&$vars)
    {
        global $textOperators;

        parent::Horde_Form($vars, _("Add Group Criterion"));

        $groups = &Group::singleton();
        $grouplist = $groups->listGroups();
        if (is_a($grouplist, 'PEAR_Error') || !count($grouplist)) {
            $type_params = array(_("Could not find any groups."));
            $this->addVariable(_("Groups"), 'groups', 'invalid', false, false, null, $type_params);
        } else {
            $this->addVariable(_("Groups"), 'groups', 'enum', true, false, null, array($grouplist));
        }
    }

    function execute(&$vars)
    {
        $path = $vars->get('path');
        $groups = $vars->get('groups');

        if ($groups) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_GROUPS, null, OPERATOR_EQUAL, $groups);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class TextCriterionForm extends Horde_Form {

    function TextCriterionForm(&$vars)
    {
        global $textOperators;

        parent::Horde_Form($vars, _("Add Text Criterion"));

        $this->addVariable(_("Text"), 'text', 'text', true);
        $this->addVariable(_("Match Operator"), 'operator', 'enum', true, false, null, array($textOperators));
        $this->addVariable(_("Search Summary"), 'summary', 'boolean', false);
        $this->addVariable(_("Search Comments"), 'comments', 'boolean', false);
    }

    function execute(&$vars)
    {
        $path = $vars->get('path');
        $text = $vars->get('text');
        $operator = $vars->get('operator');
        $summary = $vars->get('summary');
        $comments = $vars->get('comments');

        if ($summary && $comments) {
            $path = $GLOBALS['whups_query']->insertBranch($path, QUERY_TYPE_OR);
        }

        if ($summary) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_SUMMARY, null, $operator, $text);
        }

        if ($comments) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_COMMENT, null, $operator, $text);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class PropertyCriterionForm extends Horde_Form {

    function PropertyCriterionForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Add Property Criterion"));

        $this->addVariable(_("Id"), 'id', 'intlist', false);

        /* Types. */
        $this->addVariable(_("Type"), 'ttype', 'enum', false, false, null,
                           array($whups_driver->getAllTypes(), _("Any")));


        /* Queues. */
        $queues = Whups::permissionsFilter($whups_driver->getQueues(), 'queue', PERMS_READ);
        if (count($queues)) {
            $v = &$this->addVariable(_("Queue"), 'queue', 'enum', false, false,
                                     null, array($queues, _("Any")));
            $v->setAction(Horde_Form_Action::factory('reload'));
            if ($vars->get('queue')) {
                $this->addVariable(_("Version"), 'version', 'enum', false,
                                   false, null,
                                   array($whups_driver->getVersions($vars->get('queue')), _("Any")));
            }
        }

        /* States. */
        $states = $whups_driver->getStates();
        $this->addVariable(_("State"), 'state', 'enum', false, false, null,
                           array($states, _("Any")));

        /* Priorities. */
        $priorities = $whups_driver->getPriorities();
        $this->addVariable(_("Priority"), 'priority', 'enum', false, false,
                           null, array($priorities, _("Any")));
    }

    function execute(&$vars)
    {
        $path = $vars->get('path');

        $id = $vars->get('id');
        if (trim($id) != '') {
            $newpath = $path;
            $ids = split("[\\t\\n ,]+", $id);

            if (count($ids) > 1) {
                $newpath = $GLOBALS['whups_query']->insertBranch($path, QUERY_TYPE_OR);
            }

            foreach ($ids as $id) {
                $GLOBALS['whups_query']->insertCriterion($newpath, CRITERION_ID, null, OPERATOR_EQUAL, $id);
            }
        }

        $queue = $vars->get('queue');
        if ($queue) {
            $version = $vars->get('version');
            if ($version) {
                $path = $GLOBALS['whups_query']->insertBranch($path, QUERY_TYPE_AND);
            }
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_QUEUE, null, OPERATOR_EQUAL, $queue);
            if ($version) {
                $GLOBALS['whups_query']->insertCriterion($path, CRITERION_VERSION, null, OPERATOR_EQUAL, $version);
            }
        }

        $type = $vars->get('ttype');
        if ($type) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_TYPE, null, OPERATOR_EQUAL, $type);
        }

        $state = $vars->get('state');
        if ($state) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_STATE, null, OPERATOR_EQUAL, $state);
        }

        $priority = $vars->get('priority');
        if ($priority) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_PRIORITY, null, OPERATOR_EQUAL, $priority);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class AttributeCriterionForm extends Horde_Form {

    function AttributeCriterionForm(&$vars)
    {
        global $textOperators, $whups_driver;

        parent::Horde_Form($vars, _("Add Attribute Criterion"));

        // FIXME: need to search through types or otherwise list all
        // attributes.
        // $type = ?;
        // $this->attribs = $whups_driver->getAttributesForType($type);
        $this->attribs = array();

        if ($this->attribs) {
            $this->addVariable(_("Match"), 'text', 'text', true);
            $this->addVariable(_("Match Operator"), 'operator', 'enum', true, false, null, array($textOperators));

            foreach ($this->attribs as $id => $attribute) {
                $this->addVariable(sprintf(_("Search %s Attribute"), $attribute['human_name']), "a$id", 'boolean', false);
            }
        } else {
            $this->addVariable(_("Search Attribute"), 'attribute', 'invalid', true, false, null, array(_("There are no attributes defined for the current type")));
        }
    }

    function execute(&$vars)
    {
        $path = $vars->get('path');
        $text = $vars->get('text');
        $operator = $vars->get('operator');

        $count = 0;

        $keys = array_keys($this->attribs);
        foreach ($keys as $id) {
            $count += $vars->exists("a$id") ? 1 : 0;
        }

        if ($count > 1) {
            $path = $GLOBALS['whups_query']->insertBranch($path, QUERY_TYPE_OR);
        }

        foreach ($keys as $id) {
            if ($vars->get("a$id")) {
                $GLOBALS['whups_query']->insertCriterion($path, CRITERION_ATTRIBUTE, $id, $operator, $text);
            }
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class ChooseQueryNameForSaveForm extends Horde_Form {

    function ChooseQueryNameForSaveForm(&$vars)
    {
        parent::Horde_Form($vars, _("Save Query"));
        $this->setButtons(_("Save"));

        $n = &$this->addVariable(_("Name"), 'name', 'text', true);
        $n->setDefault($GLOBALS['whups_query']->name);
    }

    function execute(&$vars)
    {
        $GLOBALS['whups_query']->name = $vars->get('name');
        $result = $GLOBALS['whups_query']->save();

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class ChooseQueryNameForLoadForm extends Horde_Form {

    function ChooseQueryNameForLoadForm(&$vars)
    {
        parent::Horde_Form($vars, _("Load Query"));

        $qManager = new Whups_QueryManager();
        $qParams = $qManager->listQueries(Auth::getAuth());
        if (count($qParams)) {
            $qType = 'enum';
        } else {
            $qType = 'invalid';
            $qParams = _("You have no saved queries.");
        }

        $this->addVariable(_("Name"), 'name', $qType, true, false, null, array($qParams));
    }

    function execute(&$vars)
    {
        $qManager = new Whups_QueryManager();
        $query = $qManager->getQuery($vars->get('name'));
        if (is_a($query, 'PEAR_Error')) {
            $GLOBALS['notification']->push(sprintf(_("The query couldn't be loaded:"), $query->getMessage()), 'horde.error');
        } else {
            $GLOBALS['whups_query'] = $query;
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class DeleteQueryForm extends Horde_Form {

    function DeleteQueryForm(&$vars)
    {
        parent::Horde_Form($vars, _("Delete Query?"));

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this query? This operation is not undoable."), 'yesno', 'enum', true, false, null, $yesno);
    }

    function execute(&$vars)
    {
        global $notification;

        if ($vars->get('yesno')) {
            if (!$GLOBALS['whups_query']->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
                $notifications->push(sprintf(_("Permission denied.")), 'horde.error');
            } else {
                $result = $GLOBALS['whups_query']->delete();
                if (is_a($result, 'PEAR_Error')) {
                    $notification->push(sprintf(_("The query \"%s\" couldn't be deleted: %s"), $GLOBALS['whups_query']->name, $result->getMessage()), 'horde.error');
                } else {
                    $notification->push(sprintf(_("The query \"%s\" has been deleted."), $GLOBALS['whups_query']->name), 'horde.success');
                    $qManager = new Whups_QueryManager();
                    unset($GLOBALS['whups_query']);
                    $GLOBALS['whups_query'] = $qManager->newQuery();
                }
            }
        }

        $this->unsetVars($vars);
    }

}
